/*
    ** Open flags **

       #define PR_RDONLY       0x01
       #define PR_WRONLY       0x02
       #define PR_RDWR         0x04
       #define PR_CREATE_FILE  0x08
       #define PR_APPEND      0x10
       #define PR_TRUNCATE     0x20
       #define PR_SYNC         0x40
       #define PR_EXCL         0x80

    ** File modes **
    CAVEAT: 'mode' is currently only applicable on UNIX platforms.
    The 'mode' argument may be ignored by PR_Open on other platforms.

       00400   Read by owner.
       00200   Write by owner.
       00100   Execute (search if a directory) by owner.
       00040   Read by group.
       00020   Write by group.
       00010   Execute by group.
       00004   Read by others.
       00002   Write by others
       00001   Execute by others.
*/

function FileIO() {
}

FileIO.prototype = {
    
    //////////////////////////////
    // privates
    oFilePicker      : null,
    oUnicodeConv     : null,
    aFilters         : new Array(),
    iFilterMask      : 0,
    
    //////////////////////////////
    // publics
    consts           : Components.interfaces.nsIFilePicker,
    defaultExtension : "",
    defaultString    : "",
    
    //////////////////////////////
    // functions    
    clearFilters : function() {
        this.aFilters = new Array();
    },
    
    appendFilter : function( sTitle, sFilter ) {
        this.aFilters.push( new Array( sTitle, sFilter ) );
    },
    
    appendFiltersByMask : function( iFilterMask ) {
        this.iFilterMask |= iFilterMask;
    },
    
    pickFile : function( wndParent, sCaption, mode ) {
        if( this.oFilePicker == null ) {
            this.oFilePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
        }
        
        if( this.oFilePicker == null ) {
            return false;
        }
        
        var iRes = this.consts.returnCancel;
        try {
            this.oFilePicker.init( wndParent, sCaption, mode );
            
            this.oFilePicker.defaultExtension = this.defaultExtension;
            this.oFilePicker.defaultString    = this.defaultString;
            
            // append filters
            for( var i=0; i < this.aFilters.length; i++ ) {
                var aFilter = this.aFilters[i];
                
                if( aFilter.length == 2 ) {
                    this.oFilePicker.appendFilter(aFilter[0], aFilter[1]);
                }
            }
            
            //append filters by mask
            if( this.iFilterMask != 0 ) {
                this.oFilePicker.appendFilters( this.iFilterMask );
            }
            
            iRes = this.oFilePicker.show();
        } catch( e ) {
            
        }
        
        this.pickFileResult = iRes;
        
        switch( iRes ) {
            case this.consts.returnReplace:
            case this.consts.returnOK:
                return true;
                break;
            default:
                return false;
        }
    },
    
    getPickedFile : function() {
        if( this.oFilePicker != null ) {
            return this.oFilePicker.file;
        } else {
            return null;
        }
    },
    
    writeFile: function( sData, sCharset ) {
        if( this.oFilePicker == null || this.oFilePicker.file == null ) {
            return false;
        }
        
        var bReturn = false;
        
        try {
            if( sCharset ) {
                sData = this.fromUnicode( sData, sCharset );
            }
            
            var fOut = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
            
            fOut.init( this.oFilePicker.file, 0x02 | 0x08 | 0x20, 0664, 0 );
            fOut.write( sData, sData.length );
            fOut.close();
            
            bReturn = true;
        } catch( e ) {
            // nothing;
        }
        
        return bReturn;
    },
    
    readFile: function(sCharset) {
        if( this.oFilePicker == null || this.oFilePicker.file == null ) {
            return -1;
        }
        
        var sBuffer = new String();
        
        try {
            var fIn      = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance( Components.interfaces.nsIFileInputStream );
            var streamIn = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance( Components.interfaces.nsIScriptableInputStream );
            
            fIn.init( this.oFilePicker.file, 0x01, 0664, 0 );
            streamIn.init(fIn);
            
            sBuffer += streamIn.read(-1);
            
            streamIn.close();
            fIn.close();
            
            if( sCharset ) {
                sData = this.toUnicode( sData, sCharset );
            }
        } catch( e ) {
            //nothing
        }
        
        return sBuffer;
    },
    
    getUnicodeConverter: function() {
        if( this.oUnicodeConv == null ) {
            this.oUnicodeConv = Components.classes['@mozilla.org/intl/scriptableunicodeconverter'].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
        }
        
        return this.oUnicodeConv;
    },
    
    toUnicode : function(sData, sCharset) {
        try{
            var oUnicodeConv     = this.getUnicodeConverter();
            oUnicodeConv.charset = sCharset;
            
            sData = oUnicodeConv.ConvertToUnicode(sData);
        } catch( ex ) {
            // nothing
        }
        
        return sData;
    },
    
    fromUnicode : function(sData, sCharset) {
        try {
            var oUnicodeConv     = this.getUnicodeConverter();
            oUnicodeConv.charset = sCharset;
            
            sData = uniConv.ConvertFromUnicode(sData);
        } catch( ex ) {
            // nothing
        }
        
        return sData;
    },
}